@@ -1,11 +1,14 @@
 package Simo;
+use 5.008_001;
 use strict;
 use warnings;
+
 use Carp;
+use Simo::Error;
 
-our $VERSION = '0.0806';
+our $VERSION = '0.1001';
 
-my %VALID_IMPORT_OPT = map{ $_ => 1 } qw( base mixin );
+my %VALID_IMPORT_OPT = map{ $_ => 1 } qw( base new mixin );
 sub import{
     my ( $self, @opts ) = @_;
     @opts = %{ $opts[0] } if ref $opts[0] eq 'HASH';
@@ -24,12 +27,13 @@ sub import{
         # export function
         no strict 'refs';
         *{ "${caller_pkg}::ac" } = \&Simo::ac;
+        *{ "${caller_pkg}::and_super" } = \&Simo::and_super;
     }
     
     # caller package inherit these classes
     # 1.base class,  2.Simo,  3.mixin class
     
-    _SIMO_inherit_classes( $caller_pkg, $import_opt->{ base }, $import_opt->{ mixin } );
+    _SIMO_inherit_classes( $caller_pkg, @{ $import_opt }{ qw( base new mixin ) } );
 
     # auto strict and warnings
     strict->import;
@@ -38,9 +42,15 @@ sub import{
 
 # callar package inherit some classes
 sub _SIMO_inherit_classes{
-    my ( $pkg, $base, $mixin ) = @_;
+    my ( $pkg, $base, $new, $mixin ) = @_;
     
     my @classes;
+    
+    if( $new ){
+        push @classes,
+            ref $new eq 'ARRAY' ? @{ $new } : $new;
+    }
+    
     if( $base ){
         push @classes,
             ref $base eq 'ARRAY' ? @{ $base } : $base;
@@ -76,19 +86,85 @@ sub new{
     
     # set args
     while( my ( $attr, $val ) = splice( @args, 0, 2 ) ){
-        croak "Invalid key '$attr' is passed to new" unless $self->can( $attr );
-        
+        unless( $self->can( $attr ) ){
+            Simo::Error->throw(
+                type => 'attr_not_exist',
+                msg => "Invalid key '$attr' is passed to new",
+                pkg => $pkg,
+                attr => $attr
+            );
+        }
         no strict 'refs';
         $self->$attr( $val );
     }
     
     foreach my $required_attrs ( $self->REQUIRED_ATTRS ){
-        croak "Attr '$required_attrs' is required." 
-            unless exists $self->{ $required_attrs };
+        unless( exists $self->{ $required_attrs } ){
+            Simo::Error->throw(
+                type => 'attr_required',
+                msg => "Attr '$required_attrs' is required.",
+                pkg => $pkg,
+                attr => $required_attrs
+            );
+        }
     }
     return $self;
 }
 
+sub new_self_and_parent{
+    my $self = shift;
+    my $class = ref $self || $self;
+    
+    my $parent_pkg = do{
+        no strict 'refs';
+        ${"${class}::ISA"}[0];
+    };
+    
+    croak "Cannot call 'new_self_and_parent' from the class having no parent."
+        if $parent_pkg eq 'Simo';
+    
+    croak "'$parent_pkg' do not have 'new'." unless $parent_pkg->can( 'new' );
+    
+    my $parent;
+    my $simo;
+    
+    my $last_arg = pop;
+    if( ref $last_arg eq 'ARRAY' ){
+        my $parent_attrs = $last_arg;
+        my @args = @_;
+        
+        @args = %{ @args } if ref $args[0] eq 'HASH';
+        croak 'key-value pairs must be passed to new' if @args % 2;
+        
+        my %args = @args;
+        my %parent_args;
+        foreach my $parent_attr ( @{ $parent_attrs } ){
+            $parent_args{ $parent_attr } = delete $args{ $parent_attr };
+        }
+        
+        $parent = $parent_pkg->new( %parent_args );
+        $simo = $self->Simo::new( %args );
+    }
+    elsif( ref $last_arg eq 'HASH' && @_ == 0  ){
+        my $parent_args = $last_arg->{ parent_args };
+        my $self_args = $last_arg->{ self_args };
+
+        croak "'self_args' must be array reference." unless ref $self_args eq 'ARRAY';
+        croak "'parent_args' must be array reference." unless ref $parent_args  eq 'ARRAY';
+        
+        $parent = $parent_pkg->new( @{ $parent_args } );
+        $simo = $self->Simo::new( @{ $self_args } );
+    }
+    else{
+        croak "'new_self_and_parent' argument is invalid.";
+    }
+    
+    eval{ $parent = { %{ $parent }, %{ $simo } } };
+    croak "'$parent_pkg' must be the class based on hash reference."
+        if $@;
+    return bless $parent, $class;
+}
+
 # required keys when object is created by new.
 sub REQUIRED_ATTRS{ () }
 
@@ -190,7 +266,14 @@ sub _SIMO_create_accessor{
 
     if ( $ac_opt->{ read_only } ){
         $e .=
-        qq/    Carp::croak( "${pkg}::$attr is read only" ) if \@vals;\n\n/;
+        qq/    if( \@vals ){\n/ .
+        qq/        Simo::Error->throw(\n/ .
+        qq/            type => 'read_only',\n/ .
+        qq/            msg => "${pkg}::$attr is read only",\n/ .
+        qq/            pkg => "$pkg",\n/ .
+        qq/            attr => "$attr"\n/ .
+        qq/        );\n/ .
+        qq/    }\n\n/;
         
         goto END_OF_VALUE_SETTING;
     }
@@ -232,7 +315,13 @@ sub _SIMO_create_accessor{
         qq/            my \$ret = \$constrain->( \$val );\n/ .
         qq/            if( !\$ret ){\n/ .
         qq/                \$@ ||= 'must be valid value.';\n/ .
-        qq/                Carp::croak( "${pkg}::$attr \$@" )\n/ .
+        qq/                Simo::Error->throw(\n/ .
+        qq/                    type => 'type_invalid',\n/ .
+        qq/                    msg => "${pkg}::$attr \$@",\n/ .
+        qq/                    pkg => "$pkg",\n/ .
+        qq/                    attr => "$attr",\n/ .
+        qq/                    val => \$val\n/ .
+        qq/                );\n/ .
         qq/            }\n/ .
         qq/        }\n\n/;
     }
@@ -296,6 +385,30 @@ sub _SIMO_create_accessor{
     return $e;
 }
 
+sub and_super{
+    croak "Cannot pass args to 'and_super'" if @_;
+    my ( $self, @args );
+    my @caller;
+    {
+        package DB;
+        @caller = caller 1;
+        
+        ( $self, @args ) = @DB::args;
+    }
+    
+    my $sub = $caller[ 3 ];
+    my ( $pkg, $sub_base ) = $sub =~ /^(.*)::(.+)$/;
+    
+    my @ret;
+    {
+        no strict 'refs';
+        my $super = "SUPER::${sub_base}";
+        @ret = eval "package $pkg; \$self->\$super( \@args );";
+    }
+    if( $@ ){ $@ =~ s/\s+at .+$//; croak $@ }
+    return @ret;
+}
+
 # Helper to get acsessor info;
 sub _SIMO_get_ac_info {
     package DB;
@@ -309,7 +422,7 @@ sub _SIMO_get_ac_info {
 }
 
 ###---------------------------------------------------------------------------
-# The following methods is not recommend function 
+# The following methods is not recommended function 
 # These method is not essential as nature of object.
 # To provide the same fanctionality, I create Simo::Wrapper.
 # See Also Simo::Wrapper
@@ -319,6 +432,7 @@ sub _SIMO_get_ac_info {
 # get value specify attr names
 # ( not recommended )
 sub get_attrs{
+    carp "'get_attrs' is now not recommended. this method will be removed in future 2019/01/01";
     my ( $self, @attrs ) = @_;
     
     @attrs = @{ $attrs[0] } if ref $attrs[0] eq 'ARRAY';
@@ -335,6 +449,7 @@ sub get_attrs{
 # get value as hash specify attr names
 # ( not recommended )
 sub get_attrs_as_hash{
+    carp "'get_attrs_as_hash' is now not recommended. this method will be removed in future 2019/01/01";
     my ( $self, @attrs ) = @_;
     my @vals = $self->get_attrs( @attrs );
     
@@ -347,6 +462,7 @@ sub get_attrs_as_hash{
 # set values
 # ( not recommended )
 sub set_attrs{
+    carp "'set_attrs' is now not recommended. this method will be removed in future 2019/01/01";
     my ( $self, @args ) = @_;
 
     # check args
@@ -365,6 +481,7 @@ sub set_attrs{
 # run methods
 # ( not recommended )
 sub run_methods{
+    carp "'run_methods' is now not recommended. this method will be removed in future 2019/01/01";
     my ( $self, @method_or_args_list ) = @_;
     
     my $method_infos = $self->_SIMO_parse_run_methods_args( @method_or_args_list );
@@ -383,6 +500,7 @@ sub run_methods{
 
 # ( not recommended )
 sub _SIMO_parse_run_methods_args{
+    carp "'get_attrs' is now not recommended. this method will be removed in future 2019/01/01";
     my ( $self, @method_or_args_list ) = @_;
     
     my $method_infos = [];
@@ -410,7 +528,7 @@ Simo - Very simple framework for Object Oriented Perl.
 
 =head1 VERSION
 
-Version 0.0806
+Version 0.1001
 
 =cut
 
@@ -430,266 +548,87 @@ The feature is that
 
 =over 4
 
-=item 1. You can define accessors in very simple way.
+=item 1. You can define B<accessors> in very simple way.
 
-=item 2. Overridable new method is prepared.
+=item 2. B<new> method is prepared.
 
-=item 3. You can define default value of attribute.
+=item 3. You can define B<default value> of attribute.
 
-=item 4. Simo is very small. so You can install and excute it very fast.
+=item 4. B<Error object> is thrown, when error is occured.
 
 =back
 
 If you use Simo, you are free from bitter work 
-writing new and accessors repeatedly.
+writing new methods and accessors repeatedly.
 
 =cut
 
 =head1 SYNOPSIS
 
-=head2 Define class and accessors.
+=head2 Class definition
 
     package Book;
     use Simo;
     
-    # define accessors
     sub title{ ac }
+    sub author{ ac }
+    sub price{ ac }
+
+=head2 Using class
+
+    use Book;
+    my $book = Book->new( title => 'a', author => 'b', price => 1000 );
     
-    # define default value
+=head2 Default value of attribute
+
     sub author{ ac default => 'Kimoto' }
+
+=head2 Constraint of attribute setting
     
-    # define constrain subroutine
-    sub price{ ac constrain => sub{ /^\d+$/ } } # price must be integer.
+    use Simo::Constrain qw( is_int isa );
+    sub price{ ac constrain => sub{ is_int } }
+    sub author{ ac constrain => sub{ isa 'Person' } }
 
-    # define filter subroutine
-    sub description{ ac filter => sub{ uc } } # convert to upper case.
+=head2 Filter of attribute setting
 
-    # define trigger subroutine
-    sub issue_datetime{ ac trigger => \&update_issue_date }
-    sub issue_date{ ac } # if issue_datetime is updated, issue_date is updated.
-    
-    sub update_issue_date{
-        my $self = shift;
-        my $date = substr( $self->issue_datetime, 0, 10 );
-        $self->issue_date( $date );
-    }
-    
-    # read only accessor
-    sub get_size{ ac default => 5, read_only => 1 }
-    
-    1;
-    
-=cut
+    sub author{ ac filter => sub{ uc } }
 
-=head2 Using class and accessors
+=head2 Trigger of attribute setting
 
-    use strict;
-    use warnings;
-    use Book;
+    sub date{ ac trigger => sub{ $_->year( substr( $_->date, 0, 4 ) ) } } 
+    sub year{ ac }
 
-    # create object
-    my $book = Book->new( title => 'OO tutorial' );
+=head2 Read only accessor
 
-    # get attribute
-    my $author = $book->author;
+    sub year{ ac read_only => 1 }
 
-    # set attribute
-    $book->author( 'Ken' );
-
-    # constrain( If try to set illegal value, this call will die )
-    $book->price( 'a' ); 
-
-    # filter ( convert to 'IT IS USEFUL' )
-    $book->description( 'It is useful' );
-
-    # trigger( issue_date is updated '2009-01-01' )
-    $book->issue_datetime( '2009-01-01 12:33:45' );
-    my $issue_date = $book->issue_date;
-    
-    # read only accessor
-    $book->get_size;
-
-=cut
-
-=head1 DESCRIPTION
-
-=head2 Define class and accessors
-
-You can define class and accessors in simple way.
-
-new method is automatically created, and title accessor is defined.
-
-    package Book;
-    use Simo;
-
-    sub title{ ac }
-    1;
-
-=cut
-
-=head2 Using class and accessors
-
-You can pass key-value pairs to new, and can get and set value.
-
-    use Book;
-    
-    # create object
-    my $book = Book->new(
-        title => 'OO tutorial',
-    );
-    
-    # get value
-    my $title = $book->title;
-    
-    # set value
-    $book->title( 'The simplest OO' );
-
-=cut
-
-=head2 Automatically array convert
-
-If you pass array to accessor, array convert to array ref.
-
-    $book->title( 'a', 'b' );
-    $book->title; # get [ 'a', 'b' ], not ( 'a', 'b' )
-
-=cut
-
-=head2 Accessor options
-
-=head3 default option
-
-You can define default value of attribute.
-
-    sub title{ ac default => 'Perl is very interesting' }
-
-=cut
-
-=head3 constrain option
-
-you can constrain setting value.
-
-    sub price{ ac constrain => sub{ /^\d+$/ } }
-
-For example, If you call $book->price( 'a' ), this call is die, because 'a' is not number.
-
-'a' is set to $_. so if you can use regular expression, omit $_.
-
-you can write not omiting $_.
-
-    sub price{ ac constrain => sub{ $_ > 0 && $_ < 3 } }
-
-If you display your message when program is die, you call craok.
-    
-    use Carp;
-    sub price{ ac constrain => sub{ $_ > 0 && $_ < 3 or croak "Illegal value" } }
-
-and 'a' is alse set to first argument. So you can receive 'a' as first argument.
-
-   sub price{ ac constrain => \&_is_number }
-   
-   sub _is_number{
-       my $val = shift;
-       return $val =~ /^\d+$/;
-   }
-
-and you can define more than one constrain.
-
-    sub price{ ac constrain => [ \&_is_number, \&_is_non_zero ] }
-
-=cut
-
-=head3 filter option
-
-you can filter setting value.
-
-    sub description{ ac filter => sub{ uc } }
-
-setting value is $_ and frist argument like constrain.
-
-and you can define more than one filter.
-
-    sub description{ ac filter => [ \&uc, \&quoute ] }
-
-=cut
-
-=head3 trigger option
-
-You can define subroutine called after value is set.
-
-For example, issue_datetime is set, issue_date is update.
-
-$self is set to $_ and $_[0] different from constrain and filter.
-
-    sub issue_datetime{ ac trigger => \&update_issue_date }
-    sub issue_date{ ac }
-    
-    sub update_issue_date{
-        my $self = shift;
-        my $date = substr( $self->issue_datetime, 0, 10 );
-        $self->issue_date( $date );
-    }
-
-and you can define more than one trigger.
-
-    sub issue_datetime{ ac trigger => [ \&update_issue_date, \&update_issue_time ] }
-
-=cut
-
-=head3 read_only option
-
-Read only accessor is defined
-
-    sub get_size{ ac default => 5, read_only => 1 }
-
-Accessor name should be contain 'get_'.If not, warnings is happen.
-
-=head3 hash_force option
-
-If you pass array to accessor, Normally list convert to array ref.
-    $book->title( 'a' , 'b' ); # convert to [ 'a', 'b' ]
-
-Even if you write
-    $book->title( a => 'b' )
-
-( a => 'b' ) converted to [ 'a', 'b' ] 
-
-If you use hash_force option, you convert list to hash ref
+=head2 Hash ref convert of attribute setting
 
     sub country_id{ ac hash_force => 1 }
 
-    $book->title( a => 'b' ); # convert to { a => 'b' }
+=head2 Required attributes
+
+    sub REQUIRED_ATTRS{ qw( title author ) }
+
+=head2 Inheritance
+
+    package Magazine;
+    use Simo( base => 'Book' );
+
+=head2 Mixin
+
+    package Book;
+    use Simo( mixin => 'Class::Cloneable' );
 
 =cut
 
-=head2 Order of constrain, filter and trigger
+=head1 Manual
 
-=over 4
+See L<Simo::Manual>. 
 
-=item 1. val is passed to constrain subroutine.
+I explain detail of Simo.
 
-=item 2. val is passed to filter subroutine.
-
-=item 3. val is set
-
-=item 4. trigger subroutine is called
-
-=back
-
-       |---------|   |------|                  |-------| 
-       |         |   |      |                  |       | 
- val-->|constrain|-->|filter|-->(val is set)-->|trigger| 
-       |         |   |      |                  |       | 
-       |---------|   |------|                  |-------| 
-
-=cut
-
-=head2 Get old value
-
-You can get old value when you use accessor as setter.
-
-    $book->author( 'Ken' );
-    my $old_value = $book->author( 'Taro' ); # $old_value is 'Ken'
+If you are Japanese, See also L<Simo::Manual::Japanese>.
 
 =cut
 
@@ -704,152 +643,70 @@ ac is exported. This is used to define accessor.
     
     sub title{ ac }
     sub author{ ac }
+    sub price{ ac }
+
+=cut
+
+=head2 and_super
+
+and_super is exported. This is used to call super method for REQUIRED_ATTRS.
+
+    sub REQUIRED_ATTRS{ 'm1', 'm2', and_super }
 
 =head1 METHODS
 
 =head2 new
 
-Orveridable new method. You can create object.
+new method is prepared.
 
     use Book;
-    my $book = Book->new( title => 'a', author => 'b' );
-    
-or
+    my $book = Book->new( title => 'a', author => 'b', price => 1000 );
 
-    my $book = Book->new( { title => 'a', author => 'b' } );
+=head2 new_self_and_parent
+
+new_self_and_parent resolve the inheritance of no Simo based class;
+
+    $self->new_self_and_parent( @_, [ 'title', 'author' ] );
+    
+    $self->new_self_and_parent( { self_args => [], parent_args => [] } );
 
 =head2 REQUIRED_ATTRS
 
 this method is expected to override.
 
-You can set required attrs when object is create by new.
+You can define required attribute.
 
-    sub REQUIRED_ATTRS{ qw( title author price ) }
-
-Please be careful not to write wrong spell.
-
-=head2 get_attrs
-
-This methods is not now recommended, Please do not use this method.
-
-I realize that this method is not essential for object, 
-and become to beleve this method is helper method for object.
-
-This method will be removed in future 2019/1/1.
-
-I implement the same functionality in L<Simo::Wrapper>.
-and this is used by L<Simo::Util> o function. 
-
-Pleae see L<Simo::Util> o function.
-    
-=head2  get_attrs_as_hash
-
-This methods is not now recommended for the same reason as get_attrs.
-
-This method will be removed in future 2019/1/1.
-
-Pleae see L<Simo::Util> o function.
-
-=head2 set_attrs
-
-This methods is not now recommended for the same reason as get_attrs.
-
-This method will be removed in future 2019/1/1.
-
-Pleae see L<Simo::Util> o function.
-
-=head2 run_methods
-
-This methods is not now recommended for the same reason as get_attrs.
-
-This method will be removed in future 2019/1/1.
-
-Pleae see L<Simo::Util> o function.
-
-=head1 MORE TECHNIQUES
-
-I teach you useful techniques.
-
-=head2 New method overriding
-
-by default, new method receive key-value pairs.
-But you can change this action by overriding new method.
-
-For example, Point class. You want to call new method this way.
-
-    my $point = Point->new( 3, 5 ); # xPos and yPos
-
-You can override new method.
-    
-    package Point;
-    use Simo;
-
-    sub new{
-        my ( $self, $x, $y ) = @_; # two arg( not key-value pairs )
-        
-        # You can do anything if you need
-        
-        return $self->SUPER::new( x => $x, y => $y );
-    }
-
-    sub x{ ac }
-    sub y{ ac }
-    1;
-
-Simo implement inheritable new method.
-Whenever You change argments or add initializetion,
-You override new method.
-
-=cut
-
-=head2 Extend base class
-
-you may want to extend base class. It is OK.
-
-But I should say to you that there are one thing you should know.
-The order of Inheritance is very important.
-
-I write good sample and bad sample.
-
-    # base class
     package Book;
-    sub title{ ac };
+    use Simo;
     
-    # Good sample.
-    # inherit base class. It is OK!
-    package Magazine;
-    use base 'Book'; # use base is first
-    use Simo;        # use Simo is second;
+    sub title{ ac }
+    sub author{ ac }
+    sub price{ ac }
     
-    # Bad sample
-    package Magazine;
-    use Simo;          # use Simo is first
-    use base 'Book';   # use base is second
-
-If you call new method in Good sample, you call Book::new method.
-This is what you wanto to do.
-
-If you call new method in Bad sample, you call Simo::new method. 
-you will think why Book::new method is not called?
-
-Maybe, You will be wrong sometime. So I recomend you the following writing.
-
-    package Magazine; use base 'Book'; # package and base class
-    use Simo;                          
-
-It is like other language class Definition and I think looking is not bat.
-and you are not likely to choose wrong order.
+    sub REQUIRED_ATTRS{ qw( title author ) }
 
 =cut
 
+=head1 SEE ALSO
+
+L<Simo::Constrain> - Constraint methods for Simo 'constrain' option.
+
+L<Simo::Error> - Structured error system for Simo.
+
+L<Simo::Util> - Utitlity class for Simo. 
+
+L<Simo::Wrapper> - provide useful methods for object.
+
 =head1 CAUTION
 
-set_hook and get_hook option is now not recomended. these option will be deleted in future 2019/01/01
+B<set_hook> and B<get_hook> option is now not recommended. These option will be removed in future 2019/01/01
 
-and non named defalut value definition is not recommended. this expression cannot be available in future 2019/01/01
+B<non named defalut value definition> is now not recommended. This expression will be removed in future 2019/01/01
 
     sub title{ ac 'OO tutorial' } # not recommend. cannot be available in future.
 
+B<get_attrs>,B<get_attrs_as_hash>,B<set_attrs>,B<run_methods> is now not recommended. These methods will be removed in future 2019/01/01
+
 =cut
 
 =head1 AUTHOR
@@ -891,8 +748,7 @@ L<http://search.cpan.org/dist/Simo/>
 
 =back
 
-
-=head1 SEE ALSO
+=head1 SIMILAR MODULES
 
 L<Class::Accessor>,L<Class::Accessor::Fast>, L<Moose>, L<Mouse>.
 
